function [T, normT, difU] = lroatsym(A, r, opts)
%function [T, normT, difU] = lroatsym(A, r, opts)
%
% LROATSYM: Low Rank Orthogonal Approximation of Symmetric
% Tensors. This function computes a rank-r Kruskal symmetric tensor
% T (with orthogonal mode vectors) that best approximates the input
% symmetric tensor A.
%
% This code requires the tensor toolbox developed by Brett Bader
% and Tamara Kolda at the Sandia National Lab. The toolbox can be
% downloaded from http://csmr.ca.sandia.gov/~tgkolda/TensorToolbox/
%
% Input:
%   [A] Tensor. Datatype: tensor, ktensor or ttensor.
%   [r] Rank.
%   [opts] (Optional)
%    opts.tol: Tolerance for test of convergence. {1e-4}
%    opts.maxiter: Maximum number of iterations. {100}
%    opts.init: Initial guess. [{'nvecs'}|'random'|matrix]
%
% Output:
%   [T] The approximated tensor. Datatype: ktensor.
%       The coefficients are not guaranteed to be nonnegative,
%        but they are in decreasing order.
%   [normT] The norm of T for all iterations.
%   [difU] The differences of U between iterations.
%
% See also LROAT.

% Copyright 2009, Jie Chen.
% This program is free software; you can redistribute and/or
% modify it for NON-COMMERCIAL purposes. This program is
% distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY, including that of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.
% $Date: 2009/01/20 19:57:45$


%------------------------------------------------------------
%  preparation
%------------------------------------------------------------
% parameters
if ~exist('opts','var')      opts = struct;           end
if ~isfield(opts,'tol')      opts.tol     = 1e-4;     end
if ~isfield(opts,'maxiter')  opts.maxiter = 100;      end
if ~isfield(opts,'init')     opts.init    = 'nvecs';  end

% get dimension and order
d = size(A);                            % dimensions
N = length(d);                          % order
if r > min(d)                           % rank
  error('r larger than min{dimensions}');
end

% force the datatype of A to be `tensor'
A = full(A);
normA = norm(A);

% allocate memory
sigma = zeros(r,1);                     % sigma
V = zeros(d(1),r);                      % V matrix
Ucp = cell(1,r);                        % a copy of the U matrix
for i = 1:r                             % in a different data structure
  Ucp{i} = cell(1,N);
  for n = 1:N
    mat{i}{n} = zeros(d(n),1);
  end
end
normT = zeros(opts.maxiter,1);          % normT
difU = zeros(opts.maxiter,1);           % difU

% initial guess
if isnumeric(opts.init)
  U = opts.init;
elseif strcmp(opts.init, 'random')
  U = qr(randn(d(n),r), 0);
elseif strcmp(opts.init, 'nvecs')
  % Can set U{n} = nvecs(A, n, r);
  % But nvecs() might not return vectors in correct order
  A1 = double(tenmat(A,1));
  if r == d(n)
    [U, D] = eig(A1*A1');
  else
    eigopts.disp = 0;
    [U, D] = eigs(A1*A1', r, 'LM', eigopts);
  end
  [D, idx] = sort(diag(D), 'descend');
  U = U(:,idx);
else
  error('The initialization method is not supported.');
end

% initialize Ucp
for i = 1:r
  for n = 1:N
    mat{i}{n} = U(:,i);
  end
end


%------------------------------------------------------------
%  main iteration
%------------------------------------------------------------
for j = 1:opts.maxiter

  iter = j;

  % compute V and sigma
  % ttv is indeed slow...
  for i = 1:r
    V(:,i) = double(tenmat(ttv(A, mat{i}, -n), 1));
    sigma(i) = U(:,i)'*V(:,i);
  end
  
  % polar decomposition and update U
  % there are more sophisticated ways to do polar-decomp
  [Up, Dp, Vp] = svd(V*diag(sigma), 0);
  U1 = Up*Vp';
  difU(j) = norm(U-U1);
  U = U1;
  
  % update Ucp
  for i = 1:r
    for n = 1:N
      Ucp{i}{n} = U(:,i);
    end
  end

  % normT
  for i = 1:r
    sigma(i) = ttv(A, Ucp{i});
  end
  normT(j) = norm(sigma);

  % convergence test
  if j > 1 && norm(U-Uold) < opts.tol
    break;
  elseif j == opts.maxiter
    fprintf(1,'Maximum number of iterations (%d) reached.\n',opts.maxiter);
    fprintf(1,'Convergence has not been observed.\n');
  end
  
  % update Uold
  Uold = U;

end


%------------------------------------------------------------
%  output
%------------------------------------------------------------
% sort
[sigma, idx] = sort(sigma,'descend');
for n = 1:N
  U = U(:,idx);
end

% output
UU = cell(1,N);
for n = 1:N
  UU{n} = U;
end
T = ktensor(sigma,UU);
normT = normT(1:iter);
difU = difU(1:iter);
